function r = polyroot( this, ch, offs, range )
% determine polynomial roots
%
% r = POLYROOT( this, ch, offs )
% r = POLYROOT( this, ch, offs, range )
%
% INPUT
% this : signal (scalar object)
% ch : channel (scalar numeric)
% offs : constant offset (scalar numeric)
% range : temporal range (numeric)
%
% OUTPUT
% r : polynomial roots (column numeric)

		% safeguard
	if nargin < 1 || ~isscalar( this ) || ~isa( this, 'hNSignal' )
		error( 'invalid argument: this' );
	end

	if nargin < 2 || ~isscalar( ch ) || ~isnumeric( ch )
		error( 'invalid argument: ch' );
	end

	if nargin < 3 || ~isscalar( offs ) || ~isnumeric( offs )
		error( 'invalid argument: offs' );
	end

	if nargin < 4
		range = [1, size( this.data, 2 )-1];
	end
	if ~isnumeric( range ) || numel( range ) ~= 2
		error( 'invalid argument: range' );
	end

		% determine polynomial roots
	r = [];

	if isempty( this.coeff )
		return;
	end

	coeff = polyderiv_( this.coeff, ch-1 ); % differentiate coefficients
	coeff(1, :) = coeff(1, :) + offs/this.rate^(ch-1);

	t0 = max( 1, floor( range(1) ) ); % prepare temporal range
	t1 = min( size( this.data, 2 ), ceil( range(2) ) );

	for ti = [t0:t1]
		if ~any( isnan( coeff(:, ti) ) ) && ~any( isinf( abs( coeff(:, ti) ) ) )

				% determine polynomial roots
			lzc = roots( flipud( coeff(:, ti) ) ); % TODO: implement own version!
			lzc(imag( lzc ) ~= 0) = [];
			lzc(lzc < 0 | lzc > 1) = [];

			r(end+1:end+numel( lzc )) = lzc + ti;

				% check for roots at discontinuities
			if ti > 1
				y = polyval_( coeff(:, [ti-1, ti]), [1, 0] );
				if prod( y ) < 0
					r(end+1) = ti;
				end
			end

		end
	end

	r = sort( unique( r ) );
	r(r < range(1)) = [];
	r(r > range(2)) = [];

end % function

